@@ -13,23 +13,24 @@
 #   You should have received a copy of the GNU General Public License along
 #   with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import gi
 import os
+import subprocess
+import gi
 
 from locale import gettext as _
 
 gi.require_version('Gtk', '3.0')
 gi.require_version('Gdk', '3.0')
 
-from gi.repository import Gdk  # nopep8
-from gi.repository import GdkPixbuf  # nopep8
-from gi.repository import Gtk  # nopep8
-from gi.repository import Gio  # nopep8
-from gi.repository import GLib  # nopep8
-from gi.repository import Pango  # nopep8
+from gi.repository import Gdk  # noqa: E402
+from gi.repository import GdkPixbuf  # noqa: E402
+from gi.repository import Gtk  # noqa: E402
+from gi.repository import Gio  # noqa: E402
+from gi.repository import GLib  # noqa: E402
+from gi.repository import Pango  # noqa: E402
 
-from . import SgtSocketLauncher  # nopep8
-import sgtlauncher_lib  # nopep8
+from . import SgtSocketLauncher  # noqa: E402
+import sgtlauncher_lib  # noqa: E402
 
 
 class MyWindow(Gtk.ApplicationWindow):
@@ -46,12 +47,13 @@ class MyWindow(Gtk.ApplicationWindow):
         self.set_position(Gtk.WindowPosition.CENTER)
 
         self.launcher = SgtSocketLauncher.SgtSocketLauncher()
-        self.loading = False
-        self.load_id = 0
-        self.load_retry = 0
-        self.load_title = ""
 
-        self.socket = self.launcher.get_socket()
+        self.game_process = None
+
+        self.socket = Gtk.Socket.new()
+        self.socket.set_can_focus(True)
+        self.socket.set_focus_on_click(True)
+        self.socket.set_receives_default(True)
         self.socket.connect("plug_removed", self.socket_disconnect)
         self.socket.connect("plug_added", self.socket_connect)
 
@@ -68,6 +70,8 @@ class MyWindow(Gtk.ApplicationWindow):
         self.hb.props.title = self.title
         self.set_titlebar(self.hb)
 
+        self.set_wmclass("sgt-launcher", _("SGT Puzzles Collection"))
+
         self.stack = Gtk.Stack.new()
         self.add(self.stack)
 
@@ -144,7 +148,7 @@ class MyWindow(Gtk.ApplicationWindow):
 
         # Add the column and enable typeahead
         view.append_column(col)
-        view.set_search_column(1)
+        view.set_search_column(0)
 
         view.connect("row-activated", self.on_activated)
 
@@ -196,6 +200,12 @@ class MyWindow(Gtk.ApplicationWindow):
 
     def launch(self, title, icon_name, path):
         """Launch the specified application"""
+        # Toggle GTK_CSD=0 to guarantee embedding
+        csd_env = os.environ.copy()
+        csd_env["GTK_CSD"] = "0"
+
+        self.game_process = subprocess.Popen([path], env=csd_env)
+
         subtitle = _("Loading %s") % title
 
         if os.path.isfile(icon_name):
@@ -206,43 +216,16 @@ class MyWindow(Gtk.ApplicationWindow):
         self.launching_title.set_markup("<b>%s</b>" % title)
         self.set_view("loading", icon_name, subtitle)
 
-        self.load_icon = icon_name
-        self.load_title = title
-        self.load_retry = 0
-        self.loading_path = path
-        self.threaded_launch()
+        def on_success():
+            self.set_view("game", icon_name, title)
+        self.launcher.launch(self.socket, self.game_process,
+                             on_success, self.back_to_launcher)
 
-    def threaded_launch(self):
-        """
-        Run the application launch in a separate thread, ensuring the window
-        is correctly embedded
-        """
-        if self.loading:
-            # Application is successfully launched, switch to the game view
-            self.loading = False
-            self.load_retry = 0
-
-            # Stop the load thread
-            GLib.source_remove(self.load_id)
-            self.load_id = 0
-
-            self.set_view("game", self.load_icon, self.load_title)
-            self.grab_focus()
-            return True
-        if self.load_retry < 10:
-            # Application is not fully launched yet
-            self.loading = True
-            self.launcher.launch(self.loading_path)
-            self.load_id = GLib.timeout_add(1000, self.threaded_launch)
-            self.grab_focus()
-        else:
-            # Application failed to load, return to the launcher
-            self.loading = False
-            self.load_retry = 0
-            self.set_view("launcher")
-            self.grab_focus()
-            return True
-        return False
+    def back_to_launcher(self):
+        if self.game_process is not None:
+            self.game_process.terminate()
+            self.game_process = None
+        self.set_view("launcher")
 
     def set_view(self, name, icon_name=None, subtitle=None):
         """Change the view, setting the icon name and subtitle"""
@@ -264,7 +247,7 @@ class MyWindow(Gtk.ApplicationWindow):
         else:
             title = "%s - %s" % (title, subtitle)
 
-        if os.path.isfile(icon_name):
+        if name is not "launcher" and os.path.isfile(icon_name):
             self.set_default_icon_from_file(icon_name)
         else:
             self.set_default_icon_name(icon_name)
@@ -276,22 +259,17 @@ class MyWindow(Gtk.ApplicationWindow):
 
         if name is "game":
             self.socket.grab_focus()
+        else:
+            self.grab_focus()
 
     # Events
     def socket_connect(self, socket):
         """Embedded window connected"""
-        self.socket = self.launcher.get_socket()
         return True
 
     def socket_disconnect(self, socket):
         """Embedded window disconnected"""
-        if self.loading:
-            self.loading = False
-            self.load_retry += 1
-            GLib.source_remove(self.load_id)
-            self.load_id = GLib.timeout_add(1000, self.threaded_launch)
-        else:
-            self.set_view("launcher")
+        self.back_to_launcher()
         return True
 
     def on_keyboard_button_click(self, button, keycode, keyval):
@@ -328,19 +306,19 @@ class MyWindow(Gtk.ApplicationWindow):
 
 class MyAboutDialog(Gtk.AboutDialog):
     def __init__(self, appname, title, parent):
-        Gtk.AboutDialog.__init__(self)
+        Gtk.AboutDialog.__init__(self, use_header_bar=True)
         self.set_program_name(title)
         self.set_transient_for(parent)
         self.set_logo_icon_name(appname)
         self.set_modal(True)
 
         self.set_authors([
-            "Sean Davis (SGT Puzzles)",
+            "Sean Davis (sgt-launcher)",
             "Simon Tatham (Simon Tatham's Portable Puzzle Collection)"
         ])
         self.set_copyright(
-            "SGT Puzzles\n"
-            "© 2016-2017 Sean Davis\n"
+            "SGT Puzzles Collection (sgt-launcher)\n"
+            "© 2016-2020 Sean Davis\n"
             "\n"
             "Simon Tatham's Portable Puzzle Collection\n"
             "© 2004-2012 Simon Tatham"
@@ -349,10 +327,15 @@ class MyAboutDialog(Gtk.AboutDialog):
             "Pasi Lallinaho"
         ])
         self.set_website("https://launchpad.net/sgt-launcher")
-        self.set_website_label("SGT Puzzle Launcher on Launchpad")
         self.set_license_type(Gtk.License.GPL_3_0)
         self.set_version(sgtlauncher_lib.get_version())
 
+        # Cleanup duplicate buttons
+        hbar = self.get_header_bar()
+        for child in hbar.get_children():
+            if type(child) in [Gtk.Button, Gtk.ToggleButton]:
+                child.destroy()
+
         self.connect("response", self.on_response)
 
     def on_response(self, dialog, response):
@@ -417,31 +400,33 @@ class MyApplication(Gtk.Application):
         Gtk.Application.do_startup(self)
 
         menu = Gio.Menu()
-        # menu.append(_("Preferences"), "app.show-preferences")
         menu.append(_("About"), "app.about")
+        menu.append(_("Report a Bug..."), "app.bugreport")
         menu.append(_("Quit"), "app.quit")
         self.set_app_menu(menu)
 
-        prefs_action = Gio.SimpleAction.new("show-preferences", None)
-        prefs_action.connect("activate", self.prefs_cb)
-        self.add_action(prefs_action)
-
         about_action = Gio.SimpleAction.new("about", None)
         about_action.connect("activate", self.about_cb)
         self.add_action(about_action)
 
+        bug_action = Gio.SimpleAction.new("bugreport", None)
+        bug_action.connect("activate", self.bugreport_cb)
+        self.add_action(bug_action)
+
         quit_action = Gio.SimpleAction.new("quit", None)
         quit_action.connect("activate", self.quit_cb)
         self.add_action(quit_action)
 
-    def prefs_cb(self, action, parameter):
-        print("Not implemented")
-
     def about_cb(self, action, parameter):
         """Show the about dialog"""
         about = MyAboutDialog(self.APPNAME, self.TITLE, self.win)
         about.run()
 
+    def bugreport_cb(self, action, parameter):
+        """Take the user to the bug reporting platform"""
+        uri = "https://bugs.launchpad.net/sgt-launcher"
+        Gtk.show_uri_on_window(self.win, uri, Gdk.CURRENT_TIME)
+
     def quit_cb(self, action, parameter):
         """Exit application"""
         self.quit()
